@@ -24,6 +24,8 @@ decoder. This interface allows interpretation of tagged values.
 
 #----------------------------------------------------------------------
 
+use parent qw( CBOR::Free::Decoder::Base );
+
 use CBOR::Free ();
 
 #----------------------------------------------------------------------
@@ -36,8 +38,6 @@ Creates a new CBOR decoder object.
 
 =cut
 
-sub new { bless { _flags => 0 } }    # TODO: implement in XS, and store a context.
-
 #----------------------------------------------------------------------
 
 =head2 $data = I<OBJ>->decode( $CBOR )
@@ -62,14 +62,6 @@ references, which can cause memory leaks if not handled properly.
 
 =cut
 
-sub preserve_references {
-    if (@_ < 2 || !!$_[1]) {
-        $_[0]{'_flags'} |= _FLAG_PRESERVE_REFERENCES();
-    }
-
-    return !!($_[0]{'_flags'} & _FLAG_PRESERVE_REFERENCES());
-}
-
 #----------------------------------------------------------------------
 
 =head2 $enabled_yn = I<OBJ>->naive_utf8( [$ENABLE] )
@@ -86,16 +78,29 @@ If in doubt, leave this off.
 
 =cut
 
-sub naive_utf8 {
-    if (@_ < 2 || !!$_[1]) {
-        $_[0]{'_flags'} |= _FLAG_NAIVE_UTF8();
-    }
-
-    return !!($_[0]{'_flags'} & _FLAG_NAIVE_UTF8());
-}
-
 #----------------------------------------------------------------------
 
+=head2 $obj = I<OBJ>->string_decode_cbor();
+
+This causes I<OBJ> to decode strings according to their CBOR type:
+text strings are UTF8-decoded; binary strings are left as-is. This is
+the default configuration, à la C<CBOR::Free::decode()>.
+
+=head2 $obj = I<OBJ>->string_decode_never();
+
+This causes I<OBJ> to leave all strings undecoded. This is useful for
+applications that treat all strings as octet sequences. Note that CBOR
+text strings will still be validated as UTF-8 unless C<naive_utf8()> is
+enabled.
+
+=head2 $obj = I<OBJ>->string_decode_always();
+
+This causes I<OBJ> to decode all CBOR strings (including binary strings)
+as UTF-8, applying appropriate pre-validation unless C<naive_utf8()> is
+enabled. This is useful if you expect all strings (including binary) to be
+UTF-8 and want to handle them in Perl as character strings instead of
+byte strings.
+
 =head2 I<OBJ>->set_tag_handlers( %TAG_CALLBACK )
 
 Takes a list of key/value pairs where each key is a tag (i.e., number)
@@ -113,16 +118,4 @@ here will be ignored.
 
 =cut
 
-use constant _TAG_PACK_TMPL => eval { pack 'Q' } ? 'Q' : 'L';
-
-sub set_tag_handlers {
-    my ($self, @tag_cb) = @_;
-
-    while (my ($tag, $cb) = splice @tag_cb, 0, 2) {
-        $self->{'_tag_decode_callback'}{ pack( _TAG_PACK_TMPL(), $tag ) } = $cb;
-    }
-
-    return $self;
-}
-
 1;
